!!****if* source/physics/sourceTerms/Cool/CoolMain/Radloss/Cool
!!
!! NAME
!!
!!  Cool(integer(IN)::blockCount
!!       integer(IN)::blockList(blockCount),
!!         real(IN) :: dt,
!!         real(IN) :: time)
!!
!!
!!
!! DESCRIPTION
!!  Apply the radloss cooling opperator on the list of blocks provided as input
!!  The radiative losses rate is used to update the
!!  internal energy in the zone. The radiative losses from an 
!!  optically thin plasma are from Sarazin (1986) Rev.Mod.Phys.
!!  and from Raymond (1976).
!!  (see also Peres et al. 1982, ApJ 252, 791)
!!
!!  After we call radloss, call the eos to update the pressure 
!!  and temperature based on the radiative losses.
!!
!!
!!
!! ARGUMENTS
!!  blockList(:) : The list of blocks on which to apply the cooling operator
!!  blockCount : The number of blocks in the list
!!  dt : the current timestep
!!  time : the current time
!!
!!***


subroutine Cool(blockCount,blockList,dt, time)

!=======================================================================
  use Cool_data, ONLY : cl_Xin,cl_Abar,cl_tradmin,cl_tradmax,&
       cl_dradmin,cl_dradmax,useCool, &
       cl_h1SpecIndex, cl_elecSpecIndex
  use PhysicalConstants_interface, ONLY : PhysicalConstants_get
  use Grid_interface, ONLY : Grid_getBlkPtr, Grid_getBlkIndexLimits, &
    Grid_releaseBlkPtr
  use Eos_interface, ONLY : Eos_wrapped
  use Driver_interface, ONLY : Driver_abortFlash
#include "constants.h"
#include "Eos.h"
#include "Flash.h"

  implicit none

  integer, intent(IN) :: blockCount
  integer,dimension(blockCount), intent(IN) :: blockList
  real, intent(IN) :: dt, time
  integer :: blockID
  real, pointer, dimension(:,:,:,:) :: solnData
  integer, dimension(2,MDIM)::blkLimitsGC,blkLimits
  integer :: i, j, k, n
  
  real :: sdot, ei, ek, emag
  
  if (.not.useCool) return

  
  do blockID = 1, blockCount


     !==============================================================================
     call Grid_getBlkPtr(blockList(blockID),solnData)
     call Grid_getBlkIndexLimits(blockList(blockID),blkLimits,blkLimitsGC)

     ! sweep over all the zones
     do k = blkLimitsGC(LOW,KAXIS),blkLimitsGC(HIGH,KAXIS)
        do j = blkLimitsGC(LOW,JAXIS),blkLimitsGC(HIGH,JAXIS)
           do i = blkLimitsGC(LOW,IAXIS),blkLimitsGC(HIGH,IAXIS)

              ! change in internal energy due to radiative losses
              ek = 0.5e0*(solnData(VELX_VAR,i,j,k)**2 + &
                   solnData(VELY_VAR,i,j,k)**2 + &
                   solnData(VELZ_VAR,i,j,k)**2)

              !emag = .5*dot_product(solnData(MAGX_VAR:MAGZ_VAR,i,j,k), &
              !                      solnData(MAGX_VAR:MAGZ_VAR,i,j,k)) 

              ei = solnData(EINT_VAR,i,j,k)
              sdot = -1d-6*ei
              ei = ei + sdot

              ! update the global thermodynamic quantities due to the radiative losses
              solnData(ENER_VAR,i,j,k) = ei + ek
              solnData(EINT_VAR,i,j,k) = ei
              solnData(ENUC_VAR,i,j,k) = sdot

           enddo
        enddo
     enddo

     call Eos_wrapped(MODE_DENS_EI,blkLimits,blockID)
     call Grid_releaseBlkPtr(blockList(blockID),solnData)
  end do
  return

end subroutine Cool
